#! /usr/local/bin/perl

use warnings;
use strict;

sub usage {
    "Usage: $0 n\n" .
    "  where n is a decimal integer\n"
}

sub valid {
    my $str = shift;

    my $l = 0;
    my $r = 0;

    for (split //, $str) {
        if ($_ eq "(") {
            ++$l;
        }
        elsif ($_ eq ")") {
            ++$r;
        }

        return if $r > $l;
    }

    #print "valid [$str]: l=$l, r=$r\n";
    $l == $r;
}

sub sofar {
    my $str = shift;

    my $l = 0;
    my $r = 0;

    for (split //, $str) {
        if ($_ eq "(") {
            ++$l;
        }
        elsif ($_ eq ")") {
            ++$r;
        }

        return if $r > $l;
    }

    #print "sofar [$str]: l=$l, r=$r\n";
    $l >= $r;
}

sub balanced {
    my $n    = shift;
    my $seen = shift;
    my $acc  = shift || "";

    #print "acc = [$acc], n=$n\n";

    if (length($acc) == $n*2) {
        ++$seen->{$acc} if $acc && valid($acc);

        return;
    }

    foreach my $p ("(", ")") {
        my $try = $acc . $p;

        balanced($n,$seen,$try) if sofar $try;
    }
}

$0 =~ s,^.*[/\\],,;

my $n = @ARGV ? shift : "";

die usage unless $n =~ /^-?\d+$/;

my $seen = {};
balanced $n, $seen;

print $_, "\n" for sort keys %$seen;

