#!/usr/bin/perl

use strict;
use warnings;

my $count = shift;
die "Usage: parens PAIRS\n",
    "Prints all strings of PAIRS balanced pairs of parentheses\n"
    if @ARGV or !defined($count) or $count !~ /^\d+\z/;

$, = "\n";
print parens($count), "" if $count;

sub parens {
    my $count = $_[0] or return "";

    my @strings;

    # generate the combinations for each possible number of pairs
    # that can follow the leftmost parentheses (from the first open
    # parenthesis to its matching close parentheses.)

    for my $after (0..$count-1) {

        # determine all possible strings that can appear after
        # or within the leftmost parentheses, and generate all
        # possible combinations of the two sets

        my @after = parens($after);
        for my $within (parens($count-1-$after)) {
            push @strings, map "($within)$_", @after;
        }
    }
    @strings;
}

