#!/usr/bin/perl

use strict;
use warnings;

my $count = shift;
die "Usage: parens PAIRS\n",
    "Prints all strings of PAIRS balanced pairs of parentheses\n"
    if @ARGV or !defined($count) or $count !~ /^\d+\z/;

my @generation = [""];
strings($_, $generation[$_]) for 1..$count-1;

$\ = "\n";
tie my @out, 'Out';
strings($count, \@out);

sub strings {
    my $count = shift;

    # generate the combinations for each possible number of pairs
    # that can follow the leftmost parentheses (from the first open
    # parenthesis to its matching close parentheses.)

    for my $pairs_after (0..$count-1) {

        # determine all possible strings that can appear after
        # or within the leftmost parentheses, and generate all
        # possible combinations of the two sets

        my $after = $generation[$pairs_after];
        for my $within (@{$generation[$count-1-$pairs_after]}) {
            push @{$_[0]}, "($within)$_" for @$after;
        }
    }
}

package Out;
sub TIEARRAY { bless {} }
sub FETCHSIZE { 42 }
sub PUSH { print $_[1] }

