#!/usr/bin/perl -w
use strict;

# Iterative version

# To produce all strings of $n paren pairs, decide how many paren
# pairs go into the first paren pair and how many go outside it, and
# do the cartesian multiple.

my $n = shift;

my @parens = (['']);

# Run a code ref on all elements of a cartesian product
sub cartesian {
  my ($a, $b, $code) = @_;
  for my $in (@$a) {
    for my $out (@$b) {
      $code->("($in)$out");
    }
  }
}

for my $m (1..$n) {
  # Generate $parens[$m]
  my $code;
  if ($m < $n) {
    # In product, push onto $parens[$m]
    $code = sub { push @{$parens[$m]}, @_ };
  }
  else {
    # Last iteration - just print
    $code = sub { print "@_\n" };
  }
  for my $i (0..$m-1) {
    # Generate '(' x $parens[$m-1-$i] x ')' x $parens[$i]
    cartesian($parens[$m-$i-1], $parens[$i], $code);
  }
}
